@@ -40,19 +40,31 @@ import inspect
 import threading
 import logging
 import subprocess
+import functools
 import multiprocessing.connection
 import concurrent.futures
 from contextlib import contextmanager
 
-VERSION = "0.1.0"
+VERSION = "0.2.0"
 
 class StarryError(Exception):
     def __init__(self, output):
         self.output = output
 
+def singleton(cls):
+    instances = dict()
+
+    @functools.wraps(cls)
+    def _singleton(*args, **kwargs):
+        if cls not in instances:
+            instances[cls] = cls(*args, **kwargs)
+        return instances[cls]
+    return _singleton
+
+@singleton
 class StarryLog(object):
     def __init__(self, file=None, lvl=logging.NOTSET):
-        self.__logger = None
+        self.logger = None
 
         if os.path.isfile(file):
             try:
@@ -61,37 +73,39 @@ class StarryLog(object):
                     format=f"%(asctime)s <%(levelname)s> {inspect.stack()[-1][1].split('/')[-1]}[%(process)d]: %(message)s",
                     level=lvl)
 
-                self.__logger = logging.getLogger()
+                logging.raiseExceptions = False
+
+                self.logger = logging.getLogger()
             except:
-                self.__logger = None
+                pass
 
     def debug(self, msg):
-        if self.__logger is None:
+        if self.logger is None:
             return None
 
         try:
             caller = inspect.stack()
-            self.__logger.debug(f"[{caller[1][1].split('/')[-1]}:{caller[1][2]}] {msg}")
+            self.logger.debug(f"[{caller[1][1].split('/')[-1]}:{caller[1][2]}] {msg}")
         except:
             pass
 
     def info(self, msg):
-        if self.__logger is None:
+        if self.logger is None:
             return None
 
         try:
             caller = inspect.stack()
-            self.__logger.info(f"[{caller[1][1].split('/')[-1]}:{caller[1][2]}] {msg}")
+            self.logger.info(f"[{caller[1][1].split('/')[-1]}:{caller[1][2]}] {msg}")
         except:
             pass
 
     def warn(self, msg):
-        if self.__logger is None:
+        if self.logger is None:
             return None
 
         try:
             caller = inspect.stack()
-            self.__logger.warning(f"[{caller[1][1].split('/')[-1]}:{caller[1][2]}] {msg}")
+            self.logger.warning(f"[{caller[1][1].split('/')[-1]}:{caller[1][2]}] {msg}")
         except:
             pass
 
@@ -99,32 +113,32 @@ class StarryLog(object):
         self.warn(msg)
 
     def error(self, msg):
-        if self.__logger is None:
+        if self.logger is None:
             return None
 
         try:
             caller = inspect.stack()
-            self.__logger.error(f"[{caller[1][1].split('/')[-1]}:{caller[1][2]}] {msg}")
+            self.logger.error(f"[{caller[1][1].split('/')[-1]}:{caller[1][2]}] {msg}")
         except:
             pass
 
     def fatal(self, msg):
-        if self.__logger is None:
+        if self.logger is None:
             return None
 
         try:
             caller = inspect.stack()
-            self.__logger.critical(f"[{caller[1][1].split('/')[-1]}:{caller[1][2]}] {msg}")
+            self.logger.critical(f"[{caller[1][1].split('/')[-1]}:{caller[1][2]}] {msg}")
         except:
             pass
 
     def exception(self, msg):
-        if self.__logger is None:
+        if self.logger is None:
             return None
 
         try:
             caller = inspect.stack()
-            self.__logger.exception(f"[{caller[1][1].split('/')[-1]}:{caller[1][2]}] {msg}")
+            self.logger.exception(f"[{caller[1][1].split('/')[-1]}:{caller[1][2]}] {msg}")
         except:
             pass
 
@@ -163,7 +177,7 @@ class LibMemcachedClient(pylibmc.ThreadMappedPool):
             return mc.delete(key)
 
 class StarryServer(object):
-    def __init__(self, database, address, cache_size=0, log_file=None, rw_priority=5):
+    def __init__(self, database, address, cache_size=0, log_file=None, rw_ratio=5):
         self.database = os.path.abspath(database)
         self.address = address
         self.cache_size = cache_size
@@ -172,7 +186,7 @@ class StarryServer(object):
         self.thread_lock = threading.RLock()
         self.write_wait = 0
         self.read_wait = 0
-        self.rw_priority = rw_priority
+        self.rw_ratio = rw_ratio
         self.write_queue = []
         self.read_queue = []
         self.logger = StarryLog(log_file) if log_file is not None else None
@@ -974,7 +988,7 @@ class StarryServer(object):
     def _lock(self, mark):
         def _predicate():
             if mark["mode"] == "r":
-                if self.write_wait > 0 and (self.read_wait / self.write_wait) <= self.rw_priority:
+                if self.write_wait > 0 and (self.read_wait / self.write_wait) <= self.rw_ratio:
                     return False
 
                 if len(self.write_queue) > 0:
@@ -984,7 +998,7 @@ class StarryServer(object):
                     if any([None in write_members or len(set(write_members) & set(mark["members"])) > 0 for write_members in self.write_queue]):
                         return False
             else:
-                if self.write_wait > 0 and (self.read_wait / self.write_wait) > self.rw_priority:
+                if self.write_wait > 0 and (self.read_wait / self.write_wait) > self.rw_ratio:
                     return False
 
                 if len(self.read_queue) > 0:
